# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module containing class encapsulating Fluent connection and the Base Session."""

from enum import Enum
import json
import logging
from typing import Any, Callable, Dict
import warnings
import weakref

from ansys.fluent.core.fluent_connection import FluentConnection
from ansys.fluent.core.journaling import Journal
from ansys.fluent.core.pyfluent_warnings import (
    PyFluentDeprecationWarning,
    PyFluentUserWarning,
)
from ansys.fluent.core.services import service_creator
from ansys.fluent.core.services.app_utilities import AppUtilitiesOld
from ansys.fluent.core.services.field_data import FieldDataService, ZoneInfo
from ansys.fluent.core.services.scheme_eval import SchemeEval
from ansys.fluent.core.streaming_services.datamodel_event_streaming import (
    DatamodelEvents,
)
from ansys.fluent.core.streaming_services.events_streaming import EventsManager
from ansys.fluent.core.streaming_services.field_data_streaming import FieldDataStreaming
from ansys.fluent.core.streaming_services.transcript_streaming import Transcript
from ansys.fluent.core.utils.fluent_version import FluentVersion

from .rpvars import RPVars

try:
    from ansys.fluent.core.solver.settings import root
except Exception:
    root = Any

logger = logging.getLogger("pyfluent.general")


def _parse_server_info_file(file_name: str):
    with open(file_name, encoding="utf-8") as f:
        lines = f.readlines()
    ip_and_port = lines[0].strip().split(":")
    ip = ip_and_port[0]
    port = int(ip_and_port[1])
    password = lines[1].strip()
    return ip, port, password


class _IsDataValid:
    def __init__(self, scheme_eval):
        self._scheme_eval = scheme_eval

    def __bool__(self):
        return self()

    def __call__(self):
        return self._scheme_eval.scheme_eval("(data-valid?)")


class _AppUtilitiesFactory:
    """AppUtilities factory."""

    @staticmethod
    def _create_app_utilities(scheme_eval, fluent_connection):
        if FluentVersion(scheme_eval.version) < FluentVersion.v252:
            return AppUtilitiesOld(scheme_eval)
        else:
            return fluent_connection._connection_interface._app_utilities


class BaseSession:
    """Encapsulates a Fluent session.

    This class exposes methods for interacting with a Fluent session.

    Attributes
    ----------
    scheme_eval: SchemeEval
        Instance of ``SchemeEval`` to execute Fluent's scheme code on.

    Methods
    -------
    _create_from_server_info_file(
        server_info_file_name, cleanup_on_exit, start_transcript
        )
        Create a Session instance from server-info file

    exit()
        Close the Fluent connection and exit Fluent.
    """

    # We are passing around an WeakMethod to avoid circular references
    def __init__(
        self,
        fluent_connection: FluentConnection,
        scheme_eval: SchemeEval,
        file_transfer_service: Any | None = None,
        start_transcript: bool = True,
        launcher_args: Dict[str, Any] | None = None,
        event_type: Enum | None = None,
        get_zones_info: weakref.WeakMethod[Callable[[], list[ZoneInfo]]] | None = None,
    ):
        """BaseSession.

        Parameters
        ----------
        fluent_connection (:ref:`ref_fluent_connection`):
            Encapsulates a Fluent connection.
        scheme_eval: SchemeEval
            Instance of ``SchemeEval`` to execute Fluent's scheme code on.
        file_transfer_service : Optional
            Service for uploading and downloading files.
        start_transcript : bool, optional
            Whether to start the Fluent transcript in the client.
            The default is ``True``, in which case the Fluent
            transcript can be subsequently started and stopped
            using method calls on the ``Session`` object.
        event_type : Enum, optional
            Event enumeration specific to the session type.
        """
        self._start_transcript = start_transcript
        self._launcher_args = launcher_args
        BaseSession._build_from_fluent_connection(
            self,
            fluent_connection,
            scheme_eval,
            file_transfer_service,
            event_type,
            get_zones_info,
        )

    def _build_from_fluent_connection(
        self,
        fluent_connection: FluentConnection,
        scheme_eval: SchemeEval,
        file_transfer_service: Any | None = None,
        event_type=None,
        get_zones_info: weakref.WeakMethod[Callable[[], list[ZoneInfo]]] | None = None,
    ):
        """Build a BaseSession object from fluent_connection object."""
        self._fluent_connection = fluent_connection
        self._file_transfer_service = file_transfer_service
        self._error_state = fluent_connection._error_state
        self.scheme_eval = scheme_eval
        self.rp_vars = RPVars(self.scheme_eval.string_eval)
        self._preferences = None

        self._transcript_service = service_creator("transcript").create(
            fluent_connection._channel, fluent_connection._metadata
        )
        self.transcript = Transcript(self._transcript_service)
        if self._start_transcript:
            self.transcript.start()

        self._app_utilities = _AppUtilitiesFactory._create_app_utilities(
            self.scheme_eval, self._fluent_connection
        )

        self.journal = Journal(self._app_utilities)

        self._datamodel_service_tui = service_creator("tui").create(
            fluent_connection._channel,
            fluent_connection._metadata,
            self._error_state,
            self._app_utilities,
            self.scheme_eval,
        )

        self._datamodel_service_se = service_creator("datamodel").create(
            fluent_connection._channel,
            fluent_connection._metadata,
            self.get_fluent_version(),
            self._error_state,
            self._file_transfer_service,
        )

        self._datamodel_events = DatamodelEvents(self._datamodel_service_se)
        self._datamodel_events.start()

        self._batch_ops_service = service_creator("batch_ops").create(
            fluent_connection._channel, fluent_connection._metadata
        )

        if event_type:
            events_service = service_creator("events").create(
                fluent_connection._channel, fluent_connection._metadata
            )
            self.events = EventsManager[event_type](
                event_type, events_service, self._error_state, weakref.proxy(self)
            )
            self.events.start()
        else:
            self.events = None

        self._field_data_service = self._fluent_connection.create_grpc_service(
            FieldDataService, self._error_state
        )

        class Fields:
            """Container for field and solution variables."""

            def __init__(self, _session):
                """Initialize Fields."""
                self._is_solution_data_valid = (
                    _session._app_utilities.is_solution_data_available
                )
                self.field_info = service_creator("field_info").create(
                    _session._field_data_service,
                    self._is_solution_data_valid,
                )
                self.field_data = service_creator("field_data").create(
                    _session._field_data_service,
                    self.field_info,
                    self._is_solution_data_valid,
                    _session.scheme_eval,
                    get_zones_info,
                )
                self.field_data_streaming = FieldDataStreaming(
                    _session._fluent_connection._id, _session._field_data_service
                )
                self.field_data_old = service_creator("field_data_old").create(
                    _session._field_data_service,
                    self.field_info,
                    self._is_solution_data_valid,
                    _session.scheme_eval,
                )

        self.fields = Fields(self)

        self._settings_service = service_creator("settings").create(
            fluent_connection._channel,
            fluent_connection._metadata,
            self._app_utilities,
            self.scheme_eval,
            self._error_state,
        )

        self.health_check = fluent_connection.health_check
        self.connection_properties = fluent_connection.connection_properties

        self._fluent_connection.register_finalizer_cb(
            self._datamodel_service_se.unsubscribe_all_events
        )
        for obj in filter(None, (self._datamodel_events, self.transcript, self.events)):
            self._fluent_connection.register_finalizer_cb(obj.stop)

    @property
    def field_info(self):
        """Provides access to Fluent field information."""
        warnings.warn(
            "field_info is deprecated. Use fields.field_info instead.",
            PyFluentDeprecationWarning,
        )
        return self.fields.field_info

    @property
    def field_data(self):
        """Fluent field data on surfaces."""
        warnings.warn(
            "field_data is deprecated. Use fields.field_data instead.",
            PyFluentDeprecationWarning,
        )
        return self.fields.field_data

    @property
    def field_data_streaming(self):
        """Field gRPC streaming service of Fluent."""
        warnings.warn(
            "field_data_streaming is deprecated. Use fields.field_data_streaming instead.",
            PyFluentDeprecationWarning,
        )
        return self.fields.field_data_streaming

    @property
    def id(self) -> str:
        """Return the session ID."""
        return self._fluent_connection._id

    def start_journal(self, file_name: str):
        """Executes tui command to start journal."""
        warnings.warn("Use -> journal.start()", PyFluentDeprecationWarning)
        self.journal.start(file_name)

    def stop_journal(self):
        """Executes tui command to stop journal."""
        warnings.warn("Use -> journal.stop()", PyFluentDeprecationWarning)
        self.journal.stop()

    @classmethod
    def _create_from_server_info_file(
        cls,
        server_info_file_name: str,
        file_transfer_service: Any | None = None,
        start_transcript: bool = True,
        launcher_args: Dict[str, Any] | None = None,
        **connection_kwargs,
    ):
        """Create a Session instance from server-info file.

        Parameters
        ----------
        server_info_file_name : str
            Path to server-info file written out by Fluent server
        file_transfer_service : Optional
            Support file upload and download.
        start_transcript : bool, optional
            Whether to start the Fluent transcript in the client.
            The default is ``True``, in which case the Fluent
            transcript can be subsequently started and stopped
            using method calls on the ``Session`` object.
        **connection_kwargs : dict, optional
            Additional keyword arguments may be specified, and they will be passed to the `FluentConnection`
            being initialized. For example, ``cleanup_on_exit = True``.
            See :func:`FluentConnection initialization <ansys.fluent.core.fluent_connection.FluentConnection.__init__>`
            for more details and possible arguments.

        Returns
        -------
        Session
            Session instance
        """
        ip, port, password = _parse_server_info_file(server_info_file_name)
        fluent_connection = FluentConnection(
            ip=ip,
            port=port,
            password=password,
            file_transfer_service=file_transfer_service,
            **connection_kwargs,
        )
        session = cls(
            fluent_connection=fluent_connection,
            scheme_eval=fluent_connection._connection_interface.scheme_eval,
            file_transfer_service=file_transfer_service,
            start_transcript=start_transcript,
            launcher_args=launcher_args,
        )
        return session

    def execute_tui(self, command: str) -> None:
        """Executes a tui command."""
        self.scheme_eval.scheme_eval(f"(ti-menu-load-string {json.dumps(command)})")

    def get_fluent_version(self) -> FluentVersion:
        """Gets and returns the fluent version."""
        return FluentVersion(self.scheme_eval.version)

    def exit(self, **kwargs) -> None:
        """Exit session."""
        logger.debug("session.exit() called")
        if self._fluent_connection:
            self._fluent_connection.exit(**kwargs)

    def force_exit(self) -> None:
        """Forces the Fluent session to exit, losing unsaved progress and data."""
        self._fluent_connection.force_exit()

    def file_exists_on_remote(self, file_name: str) -> bool:
        """Check if remote file exists.

        Parameters
        ----------
        file_name: str
            File name.

        Returns
        -------
            Whether file exists.
        """
        if self._file_transfer_service:
            return self._file_transfer_service.file_exists_on_remote(file_name)

    def _file_transfer_api_warning(self, method_name: str) -> str:
        """User warning for upload/download methods."""
        return f"You have directly called the {method_name} method of the session. \
        Please be advised that for the current version of Fluent, many API methods \
        automatically handle file uploads and downloads internally. You may not \
        need to explicitly call {method_name} in most cases. \
        However, there are exceptions, particularly in PMFileManagement, where complex \
        file interactions require explicit use of {method_name}  method \
        for relevant files."

    def upload(self, file_name: list[str] | str, remote_file_name: str | None = None):
        """Upload a file to the server.

        Parameters
        ----------
        file_name : str
            Name of the local file to upload to the server.
        remote_file_name : str, optional
            remote file name, by default None
        """
        warnings.warn(self._file_transfer_api_warning("upload()"), PyFluentUserWarning)
        if self._file_transfer_service:
            return self._file_transfer_service.upload(file_name, remote_file_name)

    def download(self, file_name: str, local_directory: str | None = "."):
        """Download a file from the server.

        Parameters
        ----------
        file_name : str
            Name of the file to download from the server.
        local_directory : str, optional
            Local destination directory. The default is the current working directory.
        """
        warnings.warn(
            self._file_transfer_api_warning("download()"), PyFluentUserWarning
        )
        if self._file_transfer_service:
            return self._file_transfer_service.download(file_name, local_directory)

    def chdir(self, path: str) -> None:
        """Change Fluent working directory.

        Parameters
        ----------
        path : str
            Path of the directory to change.
        """
        self._app_utilities.set_working_directory(path)

    def __enter__(self):
        return self

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any):
        """Close the Fluent connection and exit Fluent."""
        logger.debug("session.__exit__() called")
        self.exit()

    def __dir__(self):
        dir_list = set(list(self.__dict__.keys()) + dir(type(self))) - {
            "field_data",
            "field_info",
            "field_data_streaming",
            "start_journal",
            "stop_journal",
        }
        return sorted(dir_list)



"""Solver settings."""

from ansys.fluent.core.solver.settings_builtin_bases import _SingletonSetting, _CreatableNamedObjectSetting, _NonCreatableNamedObjectSetting, Solver
from ansys.fluent.core.solver.flobject import SettingsBase


__all__ = [
    "Setup",
    "General",
    "Models",
    "Multiphase",
    "Energy",
    "Viscous",
    "Radiation",
    "Species",
    "DiscretePhase",
    "Injections",
    "Injection",
    "VirtualBladeModel",
    "Optics",
    "Structure",
    "Ablation",
    "EChemistry",
    "Battery",
    "SystemCoupling",
    "Sofc",
    "Pemfc",
    "Materials",
    "FluidMaterials",
    "FluidMaterial",
    "SolidMaterials",
    "SolidMaterial",
    "MixtureMaterials",
    "MixtureMaterial",
    "ParticleMixtureMaterials",
    "ParticleMixtureMaterial",
    "CellZoneConditions",
    "CellZoneCondition",
    "FluidCellZones",
    "FluidCellZone",
    "SolidCellZones",
    "SolidCellZone",
    "BoundaryConditions",
    "BoundaryCondition",
    "AxisBoundaries",
    "AxisBoundary",
    "DegassingBoundaries",
    "DegassingBoundary",
    "ExhaustFanBoundaries",
    "ExhaustFanBoundary",
    "FanBoundaries",
    "FanBoundary",
    "GeometryBoundaries",
    "GeometryBoundary",
    "InletVentBoundaries",
    "InletVentBoundary",
    "IntakeFanBoundaries",
    "IntakeFanBoundary",
    "InterfaceBoundaries",
    "InterfaceBoundary",
    "InteriorBoundaries",
    "InteriorBoundary",
    "MassFlowInlets",
    "MassFlowInlet",
    "MassFlowOutlets",
    "MassFlowOutlet",
    "NetworkBoundaries",
    "NetworkBoundary",
    "NetworkEndBoundaries",
    "NetworkEndBoundary",
    "OutflowBoundaries",
    "OutflowBoundary",
    "OutletVentBoundaries",
    "OutletVentBoundary",
    "OversetBoundaries",
    "OversetBoundary",
    "PeriodicBoundaries",
    "PeriodicBoundary",
    "PorousJumpBoundaries",
    "PorousJumpBoundary",
    "PressureFarFieldBoundaries",
    "PressureFarFieldBoundary",
    "PressureInlets",
    "PressureInlet",
    "PressureOutlets",
    "PressureOutlet",
    "RadiatorBoundaries",
    "RadiatorBoundary",
    "RansLesInterfaceBoundaries",
    "RansLesInterfaceBoundary",
    "RecirculationInlets",
    "RecirculationInlet",
    "RecirculationOutlets",
    "RecirculationOutlet",
    "ShadowBoundaries",
    "ShadowBoundary",
    "SymmetryBoundaries",
    "SymmetryBoundary",
    "VelocityInlets",
    "VelocityInlet",
    "WallBoundaries",
    "WallBoundary",
    "NonReflectingBoundaries",
    "NonReflectingBoundary",
    "PerforatedWallBoundaries",
    "PerforatedWallBoundary",
    "MeshInterfaces",
    "DynamicMesh",
    "ReferenceValues",
    "ReferenceFrames",
    "ReferenceFrame",
    "NamedExpressions",
    "NamedExpression",
    "Solution",
    "Methods",
    "Controls",
    "ReportDefinitions",
    "Monitor",
    "Residual",
    "ReportFiles",
    "ReportFile",
    "ReportPlots",
    "ReportPlot",
    "ConvergenceConditions",
    "CellRegisters",
    "CellRegister",
    "Initialization",
    "CalculationActivity",
    "ExecuteCommands",
    "CaseModification",
    "RunCalculation",
    "Results",
    "Surfaces",
    "PointSurfaces",
    "PointSurface",
    "LineSurfaces",
    "LineSurface",
    "RakeSurfaces",
    "RakeSurface",
    "PlaneSurfaces",
    "PlaneSurface",
    "IsoSurfaces",
    "IsoSurface",
    "IsoClips",
    "IsoClip",
    "ZoneSurfaces",
    "ZoneSurface",
    "PartitionSurfaces",
    "PartitionSurface",
    "TransformSurfaces",
    "TransformSurface",
    "ImprintSurfaces",
    "ImprintSurface",
    "PlaneSlices",
    "PlaneSlice",
    "SphereSlices",
    "SphereSlice",
    "QuadricSurfaces",
    "QuadricSurface",
    "SurfaceCells",
    "SurfaceCell",
    "ExpressionVolumes",
    "ExpressionVolume",
    "GroupSurfaces",
    "GroupSurface",
    "Graphics",
    "Meshes",
    "Mesh",
    "Contours",
    "Contour",
    "Vectors",
    "Vector",
    "Pathlines",
    "Pathline",
    "ParticleTracks",
    "ParticleTrack",
    "LICs",
    "LIC",
    "Plots",
    "XYPlots",
    "XYPlot",
    "Histogram",
    "CumulativePlots",
    "CumulativePlot",
    "ProfileData",
    "InterpolatedData",
    "Scenes",
    "Scene",
    "SceneAnimation",
    "Report",
    "DiscretePhaseHistogram",
    "Fluxes",
    "SurfaceIntegrals",
    "VolumeIntegrals",
    "InputParameters",
    "OutputParameters",
    "CustomFieldFunctions",
    "CustomFieldFunction",
    "CustomVectors",
    "CustomVector",
    "SimulationReports",
    "ParametricStudies",
    "ParametricStudy",
    "DesignPoints",
    "DesignPoint",
]

class Setup(_SingletonSetting):
    """Setup setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class General(_SingletonSetting):
    """General setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Models(_SingletonSetting):
    """Models setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Multiphase(_SingletonSetting):
    """Multiphase setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Energy(_SingletonSetting):
    """Energy setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Viscous(_SingletonSetting):
    """Viscous setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Radiation(_SingletonSetting):
    """Radiation setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Species(_SingletonSetting):
    """Species setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class DiscretePhase(_SingletonSetting):
    """DiscretePhase setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Injections(_SingletonSetting):
    """Injections setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Injection(_CreatableNamedObjectSetting):
    """Injection setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class VirtualBladeModel(_SingletonSetting):
    """VirtualBladeModel setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Optics(_SingletonSetting):
    """Optics setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Structure(_SingletonSetting):
    """Structure setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Ablation(_SingletonSetting):
    """Ablation setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class EChemistry(_SingletonSetting):
    """EChemistry setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Battery(_SingletonSetting):
    """Battery setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class SystemCoupling(_SingletonSetting):
    """SystemCoupling setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Sofc(_SingletonSetting):
    """Sofc setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Pemfc(_SingletonSetting):
    """Pemfc setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Materials(_SingletonSetting):
    """Materials setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class FluidMaterials(_SingletonSetting):
    """FluidMaterials setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class FluidMaterial(_CreatableNamedObjectSetting):
    """FluidMaterial setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class SolidMaterials(_SingletonSetting):
    """SolidMaterials setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class SolidMaterial(_CreatableNamedObjectSetting):
    """SolidMaterial setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class MixtureMaterials(_SingletonSetting):
    """MixtureMaterials setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class MixtureMaterial(_CreatableNamedObjectSetting):
    """MixtureMaterial setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ParticleMixtureMaterials(_SingletonSetting):
    """ParticleMixtureMaterials setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ParticleMixtureMaterial(_CreatableNamedObjectSetting):
    """ParticleMixtureMaterial setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class CellZoneConditions(_SingletonSetting):
    """CellZoneConditions setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CellZoneCondition(_NonCreatableNamedObjectSetting):
    """CellZoneCondition setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None):
        super().__init__(settings_source=settings_source, name=name)

class FluidCellZones(_SingletonSetting):
    """FluidCellZones setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class FluidCellZone(_CreatableNamedObjectSetting):
    """FluidCellZone setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class SolidCellZones(_SingletonSetting):
    """SolidCellZones setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class SolidCellZone(_CreatableNamedObjectSetting):
    """SolidCellZone setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class BoundaryConditions(_SingletonSetting):
    """BoundaryConditions setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class BoundaryCondition(_NonCreatableNamedObjectSetting):
    """BoundaryCondition setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None):
        super().__init__(settings_source=settings_source, name=name)

class AxisBoundaries(_SingletonSetting):
    """AxisBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class AxisBoundary(_CreatableNamedObjectSetting):
    """AxisBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class DegassingBoundaries(_SingletonSetting):
    """DegassingBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class DegassingBoundary(_CreatableNamedObjectSetting):
    """DegassingBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ExhaustFanBoundaries(_SingletonSetting):
    """ExhaustFanBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ExhaustFanBoundary(_CreatableNamedObjectSetting):
    """ExhaustFanBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class FanBoundaries(_SingletonSetting):
    """FanBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class FanBoundary(_CreatableNamedObjectSetting):
    """FanBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class GeometryBoundaries(_SingletonSetting):
    """GeometryBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class GeometryBoundary(_CreatableNamedObjectSetting):
    """GeometryBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class InletVentBoundaries(_SingletonSetting):
    """InletVentBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class InletVentBoundary(_CreatableNamedObjectSetting):
    """InletVentBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class IntakeFanBoundaries(_SingletonSetting):
    """IntakeFanBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class IntakeFanBoundary(_CreatableNamedObjectSetting):
    """IntakeFanBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class InterfaceBoundaries(_SingletonSetting):
    """InterfaceBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class InterfaceBoundary(_CreatableNamedObjectSetting):
    """InterfaceBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class InteriorBoundaries(_SingletonSetting):
    """InteriorBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class InteriorBoundary(_CreatableNamedObjectSetting):
    """InteriorBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class MassFlowInlets(_SingletonSetting):
    """MassFlowInlets setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class MassFlowInlet(_CreatableNamedObjectSetting):
    """MassFlowInlet setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class MassFlowOutlets(_SingletonSetting):
    """MassFlowOutlets setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class MassFlowOutlet(_CreatableNamedObjectSetting):
    """MassFlowOutlet setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class NetworkBoundaries(_SingletonSetting):
    """NetworkBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class NetworkBoundary(_CreatableNamedObjectSetting):
    """NetworkBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class NetworkEndBoundaries(_SingletonSetting):
    """NetworkEndBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class NetworkEndBoundary(_CreatableNamedObjectSetting):
    """NetworkEndBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class OutflowBoundaries(_SingletonSetting):
    """OutflowBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class OutflowBoundary(_CreatableNamedObjectSetting):
    """OutflowBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class OutletVentBoundaries(_SingletonSetting):
    """OutletVentBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class OutletVentBoundary(_CreatableNamedObjectSetting):
    """OutletVentBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class OversetBoundaries(_SingletonSetting):
    """OversetBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class OversetBoundary(_CreatableNamedObjectSetting):
    """OversetBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PeriodicBoundaries(_SingletonSetting):
    """PeriodicBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PeriodicBoundary(_CreatableNamedObjectSetting):
    """PeriodicBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PorousJumpBoundaries(_SingletonSetting):
    """PorousJumpBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PorousJumpBoundary(_CreatableNamedObjectSetting):
    """PorousJumpBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PressureFarFieldBoundaries(_SingletonSetting):
    """PressureFarFieldBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PressureFarFieldBoundary(_CreatableNamedObjectSetting):
    """PressureFarFieldBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PressureInlets(_SingletonSetting):
    """PressureInlets setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PressureInlet(_CreatableNamedObjectSetting):
    """PressureInlet setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PressureOutlets(_SingletonSetting):
    """PressureOutlets setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PressureOutlet(_CreatableNamedObjectSetting):
    """PressureOutlet setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class RadiatorBoundaries(_SingletonSetting):
    """RadiatorBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class RadiatorBoundary(_CreatableNamedObjectSetting):
    """RadiatorBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class RansLesInterfaceBoundaries(_SingletonSetting):
    """RansLesInterfaceBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class RansLesInterfaceBoundary(_CreatableNamedObjectSetting):
    """RansLesInterfaceBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class RecirculationInlets(_SingletonSetting):
    """RecirculationInlets setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class RecirculationInlet(_CreatableNamedObjectSetting):
    """RecirculationInlet setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class RecirculationOutlets(_SingletonSetting):
    """RecirculationOutlets setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class RecirculationOutlet(_CreatableNamedObjectSetting):
    """RecirculationOutlet setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ShadowBoundaries(_SingletonSetting):
    """ShadowBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ShadowBoundary(_CreatableNamedObjectSetting):
    """ShadowBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class SymmetryBoundaries(_SingletonSetting):
    """SymmetryBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class SymmetryBoundary(_CreatableNamedObjectSetting):
    """SymmetryBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class VelocityInlets(_SingletonSetting):
    """VelocityInlets setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class VelocityInlet(_CreatableNamedObjectSetting):
    """VelocityInlet setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class WallBoundaries(_SingletonSetting):
    """WallBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class WallBoundary(_CreatableNamedObjectSetting):
    """WallBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class NonReflectingBoundaries(_SingletonSetting):
    """NonReflectingBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class NonReflectingBoundary(_NonCreatableNamedObjectSetting):
    """NonReflectingBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None):
        super().__init__(settings_source=settings_source, name=name)

class PerforatedWallBoundaries(_SingletonSetting):
    """PerforatedWallBoundaries setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PerforatedWallBoundary(_NonCreatableNamedObjectSetting):
    """PerforatedWallBoundary setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None):
        super().__init__(settings_source=settings_source, name=name)

class MeshInterfaces(_SingletonSetting):
    """MeshInterfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class DynamicMesh(_SingletonSetting):
    """DynamicMesh setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ReferenceValues(_SingletonSetting):
    """ReferenceValues setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ReferenceFrames(_SingletonSetting):
    """ReferenceFrames setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ReferenceFrame(_CreatableNamedObjectSetting):
    """ReferenceFrame setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class NamedExpressions(_SingletonSetting):
    """NamedExpressions setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class NamedExpression(_CreatableNamedObjectSetting):
    """NamedExpression setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Solution(_SingletonSetting):
    """Solution setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Methods(_SingletonSetting):
    """Methods setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Controls(_SingletonSetting):
    """Controls setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ReportDefinitions(_SingletonSetting):
    """ReportDefinitions setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Monitor(_SingletonSetting):
    """Monitor setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Residual(_SingletonSetting):
    """Residual setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ReportFiles(_SingletonSetting):
    """ReportFiles setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ReportFile(_CreatableNamedObjectSetting):
    """ReportFile setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ReportPlots(_SingletonSetting):
    """ReportPlots setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ReportPlot(_CreatableNamedObjectSetting):
    """ReportPlot setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ConvergenceConditions(_SingletonSetting):
    """ConvergenceConditions setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CellRegisters(_SingletonSetting):
    """CellRegisters setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CellRegister(_CreatableNamedObjectSetting):
    """CellRegister setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Initialization(_SingletonSetting):
    """Initialization setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CalculationActivity(_SingletonSetting):
    """CalculationActivity setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ExecuteCommands(_SingletonSetting):
    """ExecuteCommands setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CaseModification(_SingletonSetting):
    """CaseModification setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class RunCalculation(_SingletonSetting):
    """RunCalculation setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Results(_SingletonSetting):
    """Results setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Surfaces(_SingletonSetting):
    """Surfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PointSurfaces(_SingletonSetting):
    """PointSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PointSurface(_CreatableNamedObjectSetting):
    """PointSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class LineSurfaces(_SingletonSetting):
    """LineSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class LineSurface(_CreatableNamedObjectSetting):
    """LineSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class RakeSurfaces(_SingletonSetting):
    """RakeSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class RakeSurface(_CreatableNamedObjectSetting):
    """RakeSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PlaneSurfaces(_SingletonSetting):
    """PlaneSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PlaneSurface(_CreatableNamedObjectSetting):
    """PlaneSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class IsoSurfaces(_SingletonSetting):
    """IsoSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class IsoSurface(_CreatableNamedObjectSetting):
    """IsoSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class IsoClips(_SingletonSetting):
    """IsoClips setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class IsoClip(_CreatableNamedObjectSetting):
    """IsoClip setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ZoneSurfaces(_SingletonSetting):
    """ZoneSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ZoneSurface(_CreatableNamedObjectSetting):
    """ZoneSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PartitionSurfaces(_SingletonSetting):
    """PartitionSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PartitionSurface(_CreatableNamedObjectSetting):
    """PartitionSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class TransformSurfaces(_SingletonSetting):
    """TransformSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class TransformSurface(_CreatableNamedObjectSetting):
    """TransformSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ImprintSurfaces(_SingletonSetting):
    """ImprintSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ImprintSurface(_CreatableNamedObjectSetting):
    """ImprintSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class PlaneSlices(_SingletonSetting):
    """PlaneSlices setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class PlaneSlice(_CreatableNamedObjectSetting):
    """PlaneSlice setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class SphereSlices(_SingletonSetting):
    """SphereSlices setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class SphereSlice(_CreatableNamedObjectSetting):
    """SphereSlice setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class QuadricSurfaces(_SingletonSetting):
    """QuadricSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class QuadricSurface(_CreatableNamedObjectSetting):
    """QuadricSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class SurfaceCells(_SingletonSetting):
    """SurfaceCells setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class SurfaceCell(_CreatableNamedObjectSetting):
    """SurfaceCell setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ExpressionVolumes(_SingletonSetting):
    """ExpressionVolumes setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ExpressionVolume(_CreatableNamedObjectSetting):
    """ExpressionVolume setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class GroupSurfaces(_SingletonSetting):
    """GroupSurfaces setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class GroupSurface(_CreatableNamedObjectSetting):
    """GroupSurface setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Graphics(_SingletonSetting):
    """Graphics setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Meshes(_SingletonSetting):
    """Meshes setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Mesh(_CreatableNamedObjectSetting):
    """Mesh setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Contours(_SingletonSetting):
    """Contours setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Contour(_CreatableNamedObjectSetting):
    """Contour setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Vectors(_SingletonSetting):
    """Vectors setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Vector(_CreatableNamedObjectSetting):
    """Vector setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Pathlines(_SingletonSetting):
    """Pathlines setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Pathline(_CreatableNamedObjectSetting):
    """Pathline setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ParticleTracks(_SingletonSetting):
    """ParticleTracks setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ParticleTrack(_CreatableNamedObjectSetting):
    """ParticleTrack setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class LICs(_SingletonSetting):
    """LICs setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class LIC(_CreatableNamedObjectSetting):
    """LIC setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Plots(_SingletonSetting):
    """Plots setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class XYPlots(_SingletonSetting):
    """XYPlots setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class XYPlot(_CreatableNamedObjectSetting):
    """XYPlot setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class Histogram(_SingletonSetting):
    """Histogram setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CumulativePlots(_SingletonSetting):
    """CumulativePlots setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CumulativePlot(_CreatableNamedObjectSetting):
    """CumulativePlot setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class ProfileData(_SingletonSetting):
    """ProfileData setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class InterpolatedData(_SingletonSetting):
    """InterpolatedData setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Scenes(_SingletonSetting):
    """Scenes setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Scene(_CreatableNamedObjectSetting):
    """Scene setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class SceneAnimation(_SingletonSetting):
    """SceneAnimation setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Report(_SingletonSetting):
    """Report setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class DiscretePhaseHistogram(_SingletonSetting):
    """DiscretePhaseHistogram setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class Fluxes(_SingletonSetting):
    """Fluxes setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class SurfaceIntegrals(_SingletonSetting):
    """SurfaceIntegrals setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class VolumeIntegrals(_SingletonSetting):
    """VolumeIntegrals setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class InputParameters(_SingletonSetting):
    """InputParameters setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class OutputParameters(_SingletonSetting):
    """OutputParameters setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CustomFieldFunctions(_SingletonSetting):
    """CustomFieldFunctions setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CustomFieldFunction(_CreatableNamedObjectSetting):
    """CustomFieldFunction setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class CustomVectors(_SingletonSetting):
    """CustomVectors setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class CustomVector(_CreatableNamedObjectSetting):
    """CustomVector setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class SimulationReports(_SingletonSetting):
    """SimulationReports setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ParametricStudies(_SingletonSetting):
    """ParametricStudies setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source)

class ParametricStudy(_CreatableNamedObjectSetting):
    """ParametricStudy setting."""

    def __init__(self, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name)

class DesignPoints(_SingletonSetting):
    """DesignPoints setting."""

    def __init__(self, parametric_studies: str, settings_source: SettingsBase | Solver | None = None):
        super().__init__(settings_source=settings_source, parametric_studies=parametric_studies)

class DesignPoint(_CreatableNamedObjectSetting):
    """DesignPoint setting."""

    def __init__(self, parametric_studies: str, settings_source: SettingsBase | Solver | None = None, name: str = None, new_instance_name: str = None):
        super().__init__(settings_source=settings_source, name=name, new_instance_name=new_instance_name, parametric_studies=parametric_studies)




# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a module to get global PyConsole objects."""

from ansys.fluent.core.launcher.launcher import launch_fluent
from ansys.fluent.core.session_solver import Solver


def setup_for_fluent(*args, **kwargs):
    """Returns global PyConsole objects."""
    session = launch_fluent(*args, **kwargs)
    globals = {}
    if kwargs.get("mode", "solver") == "meshing":
        globals["meshing"] = session
        globals["PartManagement"] = session.PartManagement
        globals["PMFileManagement"] = session.PMFileManagement
        globals["solver"] = Solver(
            fluent_connection=session._fluent_connection,
            scheme_eval=session._fluent_connection._connection_interface.scheme_eval,
        )
    else:
        globals["solver"] = session

    globals["preferences"] = session.preferences
    globals["workflow"] = session.workflow

    return globals



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a module for launching utilities."""

import logging
import os
from pathlib import Path
import platform
import socket
import subprocess
import time
from typing import Any, Dict

from ansys.fluent.core.exceptions import InvalidArgument
from ansys.fluent.core.utils.networking import find_remoting_ip

logger = logging.getLogger("pyfluent.launcher")


def is_windows():
    """Check if the current operating system is Windows."""
    return platform.system() == "Windows"


def _get_subprocess_kwargs_for_fluent(env: Dict[str, Any], argvals) -> Dict[str, Any]:
    import ansys.fluent.core as pyfluent

    scheduler_options = argvals.get("scheduler_options")
    is_slurm = scheduler_options and scheduler_options["scheduler"] == "slurm"
    kwargs: Dict[str, Any] = {}
    if is_slurm:
        kwargs.update(stdout=subprocess.PIPE)
    else:
        kwargs.update(
            stdout=pyfluent.LAUNCH_FLUENT_STDOUT, stderr=pyfluent.LAUNCH_FLUENT_STDERR
        )
    if is_windows():
        kwargs.update(shell=True, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP)
    else:
        kwargs.update(shell=True, start_new_session=True)
    fluent_env = os.environ.copy()
    fluent_env.update({k: str(v) for k, v in env.items()})
    fluent_env["REMOTING_THROW_LAST_TUI_ERROR"] = "1"
    if pyfluent.CLEAR_FLUENT_PARA_ENVS:
        fluent_env.pop("PARA_NPROCS", None)
        fluent_env.pop("PARA_MESH_NPROCS", None)

    if pyfluent.LAUNCH_FLUENT_IP:
        fluent_env["REMOTING_SERVER_ADDRESS"] = pyfluent.LAUNCH_FLUENT_IP

    if pyfluent.LAUNCH_FLUENT_PORT:
        fluent_env["REMOTING_PORTS"] = f"{pyfluent.LAUNCH_FLUENT_PORT}/portspan=2"

    if pyfluent.LAUNCH_FLUENT_SKIP_PASSWORD_CHECK:
        fluent_env["FLUENT_LAUNCHED_FROM_PYFLUENT"] = "1"

    if not is_slurm:
        if pyfluent.INFER_REMOTING_IP and "REMOTING_SERVER_ADDRESS" not in fluent_env:
            remoting_ip = find_remoting_ip()
            if remoting_ip:
                fluent_env["REMOTING_SERVER_ADDRESS"] = remoting_ip

    if not pyfluent.FLUENT_AUTOMATIC_TRANSCRIPT:
        fluent_env["FLUENT_NO_AUTOMATIC_TRANSCRIPT"] = "1"

    kwargs.update(env=fluent_env)
    return kwargs


def _await_fluent_launch(
    server_info_file_name: str, start_timeout: int, sifile_last_mtime: float
):
    """Wait for successful fluent launch or raise an error."""
    while True:
        if Path(server_info_file_name).stat().st_mtime > sifile_last_mtime:
            time.sleep(1)
            logger.info("Fluent has been successfully launched.")
            break
        if start_timeout == 0:
            raise TimeoutError("The launch process has timed out.")
        time.sleep(1)
        start_timeout -= 1
        logger.info("Waiting for Fluent to launch...")
        if start_timeout >= 0:
            logger.info(f"...{start_timeout} seconds remaining")


def _confirm_watchdog_start(start_watchdog, cleanup_on_exit, fluent_connection):
    """Confirm whether Fluent is running locally, and whether the Watchdog should be
    started."""
    if start_watchdog is None and cleanup_on_exit:
        host = fluent_connection.connection_properties.cortex_host
        if host == socket.gethostname():
            logger.debug(
                "Fluent running on the host machine and 'cleanup_on_exit' activated, will launch Watchdog."
            )
            start_watchdog = True
    return start_watchdog


def _build_journal_argument(
    topy: None | bool | str, journal_file_names: None | str | list[str]
) -> str:
    """Build Fluent commandline journal argument."""

    def _impl(
        topy: None | bool | str, journal_file_names: None | str | list[str]
    ) -> str:
        if journal_file_names and not isinstance(journal_file_names, (str, list)):
            raise TypeError(
                "Use 'journal_file_names' to specify and convert journal files."
            )
        if topy and not journal_file_names:
            raise InvalidArgument(
                "Use 'journal_file_names' to specify and convert journal files."
            )
        fluent_jou_arg = ""
        if isinstance(journal_file_names, str):
            journal_file_names = [journal_file_names]
        if journal_file_names:
            fluent_jou_arg += "".join(
                [f' -i "{journal}"' for journal in journal_file_names]
            )
        if topy:
            if isinstance(topy, str):
                fluent_jou_arg += f' -topy="{topy}"'
            else:
                fluent_jou_arg += " -topy"
        return fluent_jou_arg

    return _impl(topy, journal_file_names)



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from concurrent import futures
import os
from pathlib import Path
import tempfile
import time

import grpc
from grpc_health.v1 import health_pb2, health_pb2_grpc
import pytest

from ansys.api.fluent.v0 import (
    scheme_eval_pb2,
    scheme_eval_pb2_grpc,
    settings_pb2,
    settings_pb2_grpc,
)
from ansys.api.fluent.v0.scheme_pointer_pb2 import SchemePointer
import ansys.fluent.core as pyfluent
from ansys.fluent.core import connect_to_fluent, examples, session
from ansys.fluent.core.fluent_connection import FluentConnection, PortNotProvided
from ansys.fluent.core.launcher.error_handler import LaunchFluentError
from ansys.fluent.core.pyfluent_warnings import PyFluentDeprecationWarning
from ansys.fluent.core.session import BaseSession
from ansys.fluent.core.utils.execution import timeout_loop
from ansys.fluent.core.utils.file_transfer_service import RemoteFileTransferStrategy
from ansys.fluent.core.utils.fluent_version import FluentVersion
from ansys.fluent.core.utils.networking import get_free_port


class MockSettingsServicer(settings_pb2_grpc.SettingsServicer):
    def GetStaticInfo(
        self, request: settings_pb2.GetStaticInfoRequest, context: grpc.ServicerContext
    ) -> settings_pb2.GetStaticInfoResponse:
        response = settings_pb2.GetStaticInfoResponse()
        response.info.type = "Dummy"
        return response

    def GetVar(
        self,
        request: settings_pb2.GetVarRequest,
        context: grpc.ServicerContext,
    ) -> settings_pb2.GetVarResponse:
        response = settings_pb2.GetVarResponse()
        response.value.value_map.SetInParent()
        return response


class MockHealthServicer(health_pb2_grpc.HealthServicer):
    def Check(self, request, context: grpc.ServicerContext):  # noqa N802
        metadata = dict(context.invocation_metadata())
        password = metadata.get("password", None)
        if password != "12345":
            context.set_code(grpc.StatusCode.UNAUTHENTICATED)
            return health_pb2.HealthCheckResponse()
        return health_pb2.HealthCheckResponse(
            status=health_pb2.HealthCheckResponse.ServingStatus.SERVING
        )

    def Watch(self, request, context: grpc.ServicerContext):  # noqa N802
        metadata = dict(context.invocation_metadata())
        password = metadata.get("password", None)
        if password != "12345":
            context.set_code(grpc.StatusCode.UNAUTHENTICATED)
            yield health_pb2.HealthCheckResponse()

        c = 0
        while c < 2:
            time.sleep(1)
            c += 1
            yield health_pb2.HealthCheckResponse(
                status=health_pb2.HealthCheckResponse.ServingStatus.NOT_SERVING
            )

        time.sleep(1)
        yield health_pb2.HealthCheckResponse(
            status=health_pb2.HealthCheckResponse.ServingStatus.SERVING
        )


class MockSchemeEvalServicer(scheme_eval_pb2_grpc.SchemeEvalServicer):
    def StringEval(self, request, context):
        if request.input == "(cx-version)":
            return scheme_eval_pb2.StringEvalResponse(output="(23 1 0)")

    def SchemeEval(
        self,
        request,
        context: grpc.ServicerContext,
    ) -> scheme_eval_pb2.SchemeEvalResponse:
        metadata = dict(context.invocation_metadata())
        password = metadata.get("password", None)
        if password != "12345":
            context.set_code(grpc.StatusCode.UNAUTHENTICATED)
        return scheme_eval_pb2.SchemeEvalResponse(output=SchemePointer(b=True))


def test_download_file():
    with pytest.raises(examples.RemoteFileNotFoundError):
        examples.download_file(
            "mixing_elbow.cas.h5", "pyfluent/examples/DOE-ML-Mixing-Elbow"
        )


def test_create_mock_session_by_passing_ip_port_password() -> None:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    ip = "127.0.0.1"
    port = get_free_port()
    server.add_insecure_port(f"{ip}:{port}")
    health_pb2_grpc.add_HealthServicer_to_server(MockHealthServicer(), server)
    scheme_eval_pb2_grpc.add_SchemeEvalServicer_to_server(
        MockSchemeEvalServicer(), server
    )
    server.start()

    with pytest.raises(PortNotProvided):
        fluent_connection = FluentConnection(
            ip=ip, password="12345", cleanup_on_exit=False
        )
        session = BaseSession(
            fluent_connection=fluent_connection,
            scheme_eval=fluent_connection._connection_interface.scheme_eval,
        )

    fluent_connection = FluentConnection(
        ip=ip, port=port, password="12345", cleanup_on_exit=False
    )
    session = BaseSession(
        fluent_connection=fluent_connection,
        scheme_eval=fluent_connection._connection_interface.scheme_eval,
    )
    assert session.health_check.is_serving
    server.stop(None)
    session.exit()
    assert not session.health_check.is_serving


def test_create_mock_session_by_setting_ip_port_env_var(
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    ip = "127.0.0.1"
    port = get_free_port()
    server.add_insecure_port(f"{ip}:{port}")
    health_pb2_grpc.add_HealthServicer_to_server(MockHealthServicer(), server)
    scheme_eval_pb2_grpc.add_SchemeEvalServicer_to_server(
        MockSchemeEvalServicer(), server
    )
    server.start()
    monkeypatch.setenv("PYFLUENT_FLUENT_IP", ip)
    monkeypatch.setenv("PYFLUENT_FLUENT_PORT", str(port))
    fluent_connection = FluentConnection(password="12345", cleanup_on_exit=False)
    session = BaseSession(
        fluent_connection=fluent_connection,
        scheme_eval=fluent_connection._connection_interface.scheme_eval,
    )
    assert session.health_check.is_serving
    server.stop(None)
    session.exit()
    assert not session.health_check.is_serving


def test_create_mock_session_by_passing_grpc_channel() -> None:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    ip = "127.0.0.1"
    port = get_free_port()
    server.add_insecure_port(f"{ip}:{port}")
    health_pb2_grpc.add_HealthServicer_to_server(MockHealthServicer(), server)
    scheme_eval_pb2_grpc.add_SchemeEvalServicer_to_server(
        MockSchemeEvalServicer(), server
    )
    server.start()
    channel = grpc.insecure_channel(f"{ip}:{port}")
    fluent_connection = FluentConnection(
        channel=channel, cleanup_on_exit=False, password="12345"
    )
    session = BaseSession(
        fluent_connection=fluent_connection,
        scheme_eval=fluent_connection._connection_interface.scheme_eval,
    )
    assert session.health_check.is_serving
    server.stop(None)
    session.exit()
    assert not session.health_check.is_serving


def test_create_mock_session_from_server_info_file(tmp_path: Path) -> None:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    ip = "127.0.0.1"
    port = get_free_port()
    server.add_insecure_port(f"{ip}:{port}")
    health_pb2_grpc.add_HealthServicer_to_server(MockHealthServicer(), server)
    scheme_eval_pb2_grpc.add_SchemeEvalServicer_to_server(
        MockSchemeEvalServicer(), server
    )
    server.start()
    server_info_file = tmp_path / "server_info.txt"
    server_info_file.write_text(f"{ip}:{port}\n12345")
    session = BaseSession._create_from_server_info_file(
        server_info_file_name=str(server_info_file), cleanup_on_exit=False
    )
    assert session.health_check.is_serving
    server.stop(None)
    session.exit()
    assert not session.health_check.is_serving


def test_create_mock_session_from_server_info_file_with_wrong_password(
    tmp_path: Path,
) -> None:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    ip = "127.0.0.1"
    port = get_free_port()
    server.add_insecure_port(f"{ip}:{port}")
    scheme_eval_pb2_grpc.add_SchemeEvalServicer_to_server(
        MockSchemeEvalServicer(), server
    )
    health_pb2_grpc.add_HealthServicer_to_server(MockHealthServicer(), server)
    server.start()
    server_info_file = tmp_path / "server_info.txt"
    server_info_file.write_text(f"{ip}:{port}\n1234")
    with pytest.raises(RuntimeError) as ex:
        session = BaseSession._create_from_server_info_file(
            server_info_file_name=str(server_info_file),
            cleanup_on_exit=False,
        )
        session.scheme_eval.scheme_eval("")
        server.stop(None)
        session.exit()
    assert ex.value.__context__.code() == grpc.StatusCode.UNAUTHENTICATED


def test_create_mock_session_from_launch_fluent_by_passing_ip_port_password() -> None:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    ip = "127.0.0.1"
    port = get_free_port()
    server.add_insecure_port(f"{ip}:{port}")
    health_pb2_grpc.add_HealthServicer_to_server(MockHealthServicer(), server)
    scheme_eval_pb2_grpc.add_SchemeEvalServicer_to_server(
        MockSchemeEvalServicer(), server
    )
    settings_pb2_grpc.add_SettingsServicer_to_server(MockSettingsServicer(), server)
    server.start()
    session = connect_to_fluent(
        ip=ip,
        port=port,
        cleanup_on_exit=False,
        password="12345",
    )
    # check a few dir elements
    fields_dir = dir(session.fields)
    for attr in ("field_data", "field_info"):
        assert attr in fields_dir
    assert session.health_check.is_serving
    server.stop(None)
    session.exit()
    assert not session.health_check.is_serving


def test_create_mock_session_from_launch_fluent_by_setting_ip_port_env_var(
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    ip = "127.0.0.1"
    port = get_free_port()
    server.add_insecure_port(f"{ip}:{port}")
    health_pb2_grpc.add_HealthServicer_to_server(MockHealthServicer(), server)
    scheme_eval_pb2_grpc.add_SchemeEvalServicer_to_server(
        MockSchemeEvalServicer(), server
    )
    settings_pb2_grpc.add_SettingsServicer_to_server(MockSettingsServicer(), server)
    server.start()
    monkeypatch.setenv("PYFLUENT_FLUENT_IP", ip)
    monkeypatch.setenv("PYFLUENT_FLUENT_PORT", str(port))
    session = connect_to_fluent(
        cleanup_on_exit=False, ip=ip, port=port, password="12345"
    )
    # check a few dir elements
    fields_dir = dir(session.fields)
    for attr in ("field_data", "field_info"):
        assert attr in fields_dir
    assert session.health_check.is_serving
    server.stop(None)
    session.exit()
    assert not session.health_check.is_serving


@pytest.mark.parametrize("file_format", ["jou", "py"])
@pytest.mark.fluent_version(">=23.2")
def test_journal_creation(file_format, new_meshing_session):
    fd, file_name = tempfile.mkstemp(
        suffix=f"-{os.getpid()}.{file_format}",
        prefix="pyfluent-",
        dir=str(pyfluent.EXAMPLES_PATH),
    )
    os.close(fd)

    file_name = Path(file_name)

    file_name.touch()
    prev_stat = file_name.stat()
    prev_mtime = prev_stat.st_mtime
    prev_size = prev_stat.st_size
    print(f"prev_stat: {prev_stat}")

    session = new_meshing_session
    if session.connection_properties.inside_container:
        session.journal.start(file_name.name)
    else:
        session.journal.start(str(file_name))
    session = session.switch_to_solver()
    session.journal.stop()
    new_stat = file_name.stat()
    print(f"new_stat: {new_stat}")
    assert new_stat.st_mtime > prev_mtime or new_stat.st_size > prev_size


@pytest.mark.fluent_version(">=23.2")
def test_start_transcript_file_write(new_meshing_session):
    fd, file_name = tempfile.mkstemp(
        suffix=f"-{os.getpid()}.trn",
        prefix="pyfluent-",
        dir=str(pyfluent.EXAMPLES_PATH),
    )
    os.close(fd)

    file_name = Path(file_name)

    file_name.touch()
    # prev_stat = file_name.stat()
    # prev_mtime = prev_stat.st_mtime
    # prev_size = prev_stat.st_size

    session = new_meshing_session
    session.transcript.start(file_name)
    session = session.switch_to_solver()
    session.transcript.stop()

    # new_stat = file_name.stat()
    # this assertion is invalid.
    # assert new_stat.st_mtime > prev_mtime or new_stat.st_size > prev_size


@pytest.mark.fluent_version(">=23.1")
def test_expected_interfaces_in_solver_session(new_solver_session):
    assert all(
        intf in dir(new_solver_session) for intf in ("preferences", "tui", "workflow")
    )


@pytest.mark.fluent_version(">=24.1")
def test_solverworkflow_not_in_solver_session(new_solver_session):
    assert "solverworkflow" not in dir(new_solver_session)


@pytest.mark.standalone
@pytest.mark.fluent_version(">=23.2")
def test_read_case_using_lightweight_mode():
    import_file_name = examples.download_file(
        "mixing_elbow.cas.h5", "pyfluent/mixing_elbow"
    )
    if pyfluent.USE_FILE_TRANSFER_SERVICE:
        file_transfer_service = RemoteFileTransferStrategy()
        container_dict = {"mount_source": file_transfer_service.MOUNT_SOURCE}
        solver = pyfluent.launch_fluent(
            case_file_name=import_file_name,
            lightweight_mode=True,
            container_dict=container_dict,
            file_transfer_service=file_transfer_service,
        )
    else:
        solver = pyfluent.launch_fluent(
            case_file_name=import_file_name, lightweight_mode=True
        )
    solver.setup.models.energy.enabled = False
    old_fluent_connection_id = id(solver._fluent_connection)
    timeout_loop(
        id(solver._fluent_connection) != old_fluent_connection_id,
        timeout=60,
        idle_period=1,
    )
    timeout_loop(
        not solver.setup.models.energy.enabled(),
        timeout=60,
        idle_period=1,
    )
    solver.exit()


def test_help_does_not_throw(new_solver_session):
    help(new_solver_session.file.read)


@pytest.fixture
def new_solver_session2(new_solver_session):
    return new_solver_session


def test_build_from_fluent_connection(new_solver_session, new_solver_session2):
    solver1 = new_solver_session
    solver2 = new_solver_session2
    assert solver1.health_check.is_serving
    assert solver2.health_check.is_serving
    health_check_service1 = solver1.health_check
    cortex_pid2 = solver2._fluent_connection.connection_properties.cortex_pid
    # The below hack is performed to check the base class method
    # (child class has a method with same name)
    solver1.__class__.__bases__[0]._build_from_fluent_connection(
        solver1,
        fluent_connection=solver2._fluent_connection,
        scheme_eval=solver2._fluent_connection._connection_interface.scheme_eval,
    )
    assert solver1.health_check.is_serving
    assert solver2.health_check.is_serving
    timeout_loop(
        not health_check_service1.is_serving,
        timeout=60,
        idle_period=1,
    )
    assert solver1._fluent_connection.connection_properties.cortex_pid == cortex_pid2
    assert solver2._fluent_connection.connection_properties.cortex_pid == cortex_pid2


@pytest.mark.standalone
def test_recover_grpc_error_from_launch_error(monkeypatch: pytest.MonkeyPatch):
    orig_parse_server_info_file = session._parse_server_info_file

    def mock_parse_server_info_file(file_name):
        ip, port, password = orig_parse_server_info_file(file_name)
        return ip, port - 1, password  # provide wrong port

    monkeypatch.setattr(session, "_parse_server_info_file", mock_parse_server_info_file)
    with pytest.raises(LaunchFluentError) as ex:
        _ = pyfluent.launch_fluent()
    # grpc.RpcError -> RuntimeError -> LaunchFluentError
    assert ex.value.__context__.__context__.code() == grpc.StatusCode.UNAVAILABLE


def test_solver_methods(new_solver_session):
    solver = new_solver_session

    if solver.get_fluent_version() == FluentVersion.v222:
        api_keys = {
            "file",
            "setup",
            "solution",
            "results",
            "parametric_studies",
            "current_parametric_study",
        }
        assert api_keys.issubset(set(dir(solver)))
    if solver.get_fluent_version() == FluentVersion.v232:
        api_keys = {
            "file",
            "mesh",
            "server",
            "setup",
            "solution",
            "results",
            "parametric_studies",
            "current_parametric_study",
            "parallel",
            "report",
        }
        assert api_keys.issubset(set(dir(solver)))
    if solver.get_fluent_version() >= FluentVersion.v241:
        api_keys = {
            "file",
            "mesh",
            "server",
            "setup",
            "solution",
            "results",
            "parametric_studies",
            "current_parametric_study",
            "parallel",
        }
        if solver.get_fluent_version() >= FluentVersion.v251:
            assert api_keys.issubset(set(dir(solver.settings)))
        else:
            assert api_keys.issubset(set(dir(solver)))


@pytest.mark.fluent_version(">=23.2")
def test_get_set_state_on_solver(new_solver_session):
    solver = new_solver_session
    state = solver.get_state()
    assert state
    solver.set_state(state)


def test_solver_structure(new_solver_session):
    solver = new_solver_session
    with pytest.warns(PyFluentDeprecationWarning):
        solver.field_data
    with pytest.warns(PyFluentDeprecationWarning):
        solver.svar_data

    assert {
        "field_data",
        "field_info",
        "field_data_streaming",
        "solution_variable_data",
        "solution_variable_info",
        "reduction",
    }.issubset(set(dir(solver.fields)))


@pytest.mark.fluent_version(">=24.2")
def test_general_exception_behaviour_in_session(new_solver_session):
    solver = new_solver_session

    # Read case with non-existent path
    with pytest.raises(RuntimeError):
        # File not found
        solver.settings.file.read(
            file_type="case", file_name=r"incorrect_path\incorrect_file.cas.h5"
        )

    # Iterate with no case
    with pytest.raises(RuntimeError):
        # The object is not active
        solver.solution.run_calculation.iterate(iter_count=5)

    # Write case without any case loaded or created
    with pytest.raises(RuntimeError):
        # Uninitialized case
        solver.file.write(file_name="sample.cas.h5", file_type="case")

    graphics = solver.results.graphics

    # # Post-process without case
    # with pytest.raises(RuntimeError):
    #     # Does not exist. #1197711
    #     graphics.mesh["mesh-1"] = {"surfaces_list": "*"}
    #     graphics.mesh["mesh-1"].display()

    case_file = examples.download_file(
        "mixing_elbow.cas.h5",
        "pyfluent/mixing_elbow",
        return_without_path=False,
    )
    solver.settings.file.read(file_type="case", file_name=case_file)
    solver.file.write(file_name="sample.cas.h5", file_type="case")

    graphics.mesh["mesh-1"] = {"surfaces_list": "*"}
    graphics.mesh["mesh-1"].display()

    # Post-process without data
    with pytest.raises(RuntimeError):
        # Invalid result.
        graphics.contour["contour-velocity"] = {
            "field": "velocity-magnitude",
            "surfaces_list": ["wall-elbow"],
        }
        graphics.contour["contour-velocity"].display()

    solver.solution.run_calculation.iterate(iter_count=5)
    graphics.contour["contour-velocity"] = {
        "field": "velocity-magnitude",
        "surfaces_list": ["wall-elbow"],
    }
    graphics.contour["contour-velocity"].display()

    mesh_file_2d = examples.download_file(
        "sample_2d_mesh.msh.h5",
        "pyfluent/surface_mesh",
        return_without_path=False,
    )

    # Error in server:
    # This appears to be a surface mesh.\nSurface meshes cannot be read under the /file/read-case functionality.
    # with pytest.raises(RuntimeError):
    #     solver.settings.file.read(file_type='case', file_name=mesh_file_2d)
    del mesh_file_2d  # Remove this line after the above code snippet is operational


@pytest.mark.fluent_version(">=23.2")
def test_app_utilities_new_and_old(mixing_elbow_settings_session):
    solver = mixing_elbow_settings_session

    assert solver._app_utilities.get_app_mode() == pyfluent.FluentMode.SOLVER

    assert not solver._app_utilities.is_beta_enabled()

    assert not solver._app_utilities.is_wildcard("no")

    assert solver._app_utilities.is_wildcard("yes*")

    assert not solver._app_utilities.is_solution_data_available()

    tmp_dir = tempfile.mkdtemp(dir=pyfluent.EXAMPLES_PATH)

    solver.chdir(tmp_dir)

    assert Path(
        solver._app_utilities.get_controller_process_info()["working_directory"]
    ) == Path(tmp_dir)

    assert Path(
        solver._app_utilities.get_solver_process_info()["working_directory"]
    ) == Path(tmp_dir)



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""A module for controlling the writing of Fluent Python journals."""


class PythonJournalNotSupported(RuntimeError):
    """Raised when Python journal is unsupported."""

    def __init__(self):
        """Initialize PythonJournalNotSupported."""
        super().__init__(
            "Python journaling is available in Fluent version 2023 R1 or later."
        )


class Journal:
    """Control the writing of Fluent Python journals."""

    def __init__(self, app_utilities):
        """__init__ method of Journal class."""
        self._app_utilities = app_utilities

    def _check_python_journaling_support(self):
        if self._app_utilities.get_product_version() == "22.2.0":
            raise PythonJournalNotSupported()

    def start(self, file_name: str):
        """Start writing a Fluent Python journal at the specified file_name."""
        self._check_python_journaling_support()
        self._app_utilities.start_python_journal(journal_name=file_name)

    def stop(self):
        """Stop writing the Fluent Python journal."""
        self._check_python_journaling_support()
        self._app_utilities.stop_python_journal()



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""".. _conjugate_heat_transfer:

Conjugate Heat Transfer
-----------------------
"""

#######################################################################################
# Objective
# =====================================================================================
#
# This tutorial demonstrates how to model forced convection in a louvered fin heat
# exchanger. This case solves equations for both Fluid and Solid domain.
# As a result, temperature field evolved together.
#
# This tutorial demonstrates how to perform the following tasks:
#
# * Calculate the fin heat transfer rate.
# * Use periodic boundaries to reduce the size of the computational domain.
# * Use a convective thermal boundary condition to represent heat transfer.
# * Examine and understand the relationship between flow and temperature.

# sphinx_gallery_thumbnail_path = '_static/cht_xy_pressure.png'

###################################
# Import required libraries/modules
# =================================

import csv
import os
from pathlib import Path

import matplotlib.pyplot as plt
import pyvista as pv

import ansys.fluent.core as pyfluent
from ansys.fluent.core import examples
from ansys.fluent.visualization.matplotlib import Plots
from ansys.fluent.visualization.pyvista import Graphics, pyvista_windows_manager
from ansys.fluent.visualization.pyvista.pyvista_windows_manager import PyVistaWindow

geom_filename = examples.download_file(
    "cht_fin_htc_new.scdoc",
    "pyfluent/examples/CHT",
)

#######################
# Fluent Solution Setup
# =====================

##################################################################
# Launch Fluent session with meshing mode and print Fluent version
# ================================================================

meshing = pyfluent.launch_fluent(
    product_version="25.1.0",
    mode="meshing",
    dimension=3,
    precision="double",
    processor_count=4,
)
print(meshing.get_fluent_version())

#############################################################################
# Start Watertight Geometry Meshing Workflow
# ==========================================

meshing.workflow.InitializeWorkflow(WorkflowType=r"Watertight Geometry")

meshing.workflow.TaskObject["Import Geometry"].Arguments = dict(FileName=geom_filename)

meshing.workflow.TaskObject["Import Geometry"].Execute()

meshing.workflow.TaskObject["Add Local Sizing"].Execute()

meshing.workflow.TaskObject["Generate the Surface Mesh"].Arguments = dict(
    {
        "CFDSurfaceMeshControls": {
            "MinSize": 0.3,
            "MaxSize": 1,
            "ScopeProximityTo": "faces",
        },
    }
)
meshing.workflow.TaskObject["Generate the Surface Mesh"].Execute()

meshing.workflow.TaskObject["Describe Geometry"].UpdateChildTasks(SetupTypeChanged=True)
meshing.workflow.TaskObject["Describe Geometry"].Arguments.setState(
    {
        r"CappingRequired": r"No",
        r"InvokeShareTopology": r"No",
        r"NonConformal": r"Yes",
        r"SetupType": r"The geometry consists of both fluid and solid regions and/or voids",
    }
)

meshing.workflow.TaskObject["Describe Geometry"].Execute()

#############################################################################
# Update Interface Boundaries; Create Region
# ==========================================

meshing.workflow.TaskObject["Update Boundaries"].Arguments.setState(
    {
        r"BoundaryLabelList": [
            r"interface-out-solid-a",
            r"interface-out-high-a",
            r"interface-out-low-a",
            r"interface-4-solid-sweep",
            r"interface-4-high-sweep",
            r"interface-4-low-sweep",
            r"interface-3-solid-sweep",
            r"interface-3-high-sweep",
            r"interface-3-low-sweep",
            r"interface-2-solid-sweep",
            r"interface-2-high-sweep",
            r"interface-2-low-sweep",
            r"interface-1-solid-sweep",
            r"interface-1-high-sweep",
            r"interface-1-low-sweep",
            r"interface-solid-in-a",
            r"interface-in-high-a",
            r"interface-in-low-a",
            r"interface-tube-2-solid-a",
            r"interface-tube-2-high-a",
            r"interface-tube-2-low-a",
            r"interface-tube-1-solid-a",
            r"interface-tube-1-high-a",
            r"interface-tube-1-low-a",
            r"interface-4-fluid-high-tet",
            r"interface-4-fluid-low-tet",
            r"interface-3-fluid-low-tet",
            r"interface-3-fluid-high-tet",
            r"interface-2-fluid-high-tet",
            r"interface-2-fluid-low-tet",
            r"interface-1-fluid-high-tet",
            r"interface-1-fluid-low-tet",
            r"interface-1-solid-tet-4",
            r"interface-1-solid-tet-3",
            r"interface-1-solid-tet-2",
            r"interface-1-solid-tet-1",
        ],
        r"BoundaryLabelTypeList": [
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
            r"interface",
        ],
        r"OldBoundaryLabelList": [
            r"interface-out-solid-a",
            r"interface-out-high-a",
            r"interface-out-low-a",
            r"interface-4-solid-sweep",
            r"interface-4-high-sweep",
            r"interface-4-low-sweep",
            r"interface-3-solid-sweep",
            r"interface-3-high-sweep",
            r"interface-3-low-sweep",
            r"interface-2-solid-sweep",
            r"interface-2-high-sweep",
            r"interface-2-low-sweep",
            r"interface-1-solid-sweep",
            r"interface-1-high-sweep",
            r"interface-1-low-sweep",
            r"interface-solid-in-a",
            r"interface-in-high-a",
            r"interface-in-low-a",
            r"interface-tube-2-solid-a",
            r"interface-tube-2-high-a",
            r"interface-tube-2-low-a",
            r"interface-tube-1-solid-a",
            r"interface-tube-1-high-a",
            r"interface-tube-1-low-a",
            r"interface-4-fluid-high-tet",
            r"interface-4-fluid-low-tet",
            r"interface-3-fluid-low-tet",
            r"interface-3-fluid-high-tet",
            r"interface-2-fluid-high-tet",
            r"interface-2-fluid-low-tet",
            r"interface-1-fluid-high-tet",
            r"interface-1-fluid-low-tet",
            r"interface-1-solid-tet-4",
            r"interface-1-solid-tet-3",
            r"interface-1-solid-tet-2",
            r"interface-1-solid-tet-1",
        ],
        r"OldBoundaryLabelTypeList": [
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
            r"wall",
        ],
        r"OldLabelZoneList": [
            r"interface-out-solid-a",
            r"interface-out-high-a",
            r"interface-out-low-a",
            r"interface-4-solid-sweep",
            r"interface-4-high-sweep",
            r"interface-4-low-sweep",
            r"interface-3-solid-sweep",
            r"interface-3-high-sweep",
            r"interface-3-low-sweep",
            r"interface-2-solid-sweep",
            r"interface-2-high-sweep",
            r"interface-2-low-sweep",
            r"interface-1-solid-sweep",
            r"interface-1-high-sweep",
            r"interface-1-low-sweep",
            r"interface-solid-in-a",
            r"interface-in-high-a",
            r"interface-in-low-a",
            r"interface-tube-2-solid-a.2",
            r"interface-tube-2-solid-a.1",
            r"interface-tube-2-solid-a",
            r"interface-tube-2-high-a.2",
            r"interface-tube-2-high-a.1",
            r"interface-tube-2-high-a",
            r"interface-tube-2-low-a.2",
            r"interface-tube-2-low-a.1",
            r"interface-tube-2-low-a",
            r"interface-tube-1-solid-a.2",
            r"interface-tube-1-solid-a.1",
            r"interface-tube-1-solid-a",
            r"interface-tube-1-high-a.2",
            r"interface-tube-1-high-a.1",
            r"interface-tube-1-high-a",
            r"interface-tube-1-low-a.2",
            r"interface-tube-1-low-a.1",
            r"interface-tube-1-low-a",
            r"interface-4-fluid-high-tet",
            r"interface-4-fluid-low-tet",
            r"interface-3-fluid-low-tet",
            r"interface-3-fluid-high-tet",
            r"interface-2-fluid-high-tet",
            r"interface-2-fluid-low-tet",
            r"interface-1-fluid-high-tet",
            r"interface-1-fluid-low-tet",
            r"interface-1-solid-tet-4",
            r"interface-1-solid-tet-3",
            r"interface-1-solid-tet-2",
            r"interface-1-solid-tet-1",
        ],
    }
)

meshing.workflow.TaskObject["Update Boundaries"].Execute()

meshing.workflow.TaskObject["Create Regions"].Execute()

#############################################################################
# Custom Journal for Creating Periodicity due to Non-Conformal Objects
# ====================================================================

meshing.workflow.TaskObject["Describe Geometry"].InsertNextTask(
    CommandName=r"RunCustomJournal"
)
meshing.workflow.TaskObject["Run Custom Journal"].Rename(NewName=r"set-periodicity")
meshing.workflow.TaskObject["set-periodicity"].Arguments = dict(
    {
        r"JournalString": r"""/bo rps translational semi-auto periodic-1-high periodic-2-high periodic-3-high periodic-4-high , 0 0 -2.3
/bo rps translational semi-auto periodic-5* , 0 0 -2.3
/bo rps translational semi-auto periodic-6-high , 0 0 -2.3
/bo rps translational semi-auto periodic-7-high , 0 0 -2.3
""",
    }
)

meshing.workflow.TaskObject["set-periodicity"].Execute()

#############################################################################
# Update Boundary Layer Task
# ==========================

meshing.workflow.TaskObject["Update Regions"].Execute()
meshing.workflow.TaskObject["Add Boundary Layers"].AddChildToTask()
meshing.workflow.TaskObject["Add Boundary Layers"].InsertCompoundChildTask()
meshing.workflow.TaskObject["smooth-transition_1"].Rename(NewName=r"aspect-ratio_1")

meshing.workflow.TaskObject["aspect-ratio_1"].Arguments.setState(
    {
        "BLControlName": r"aspect-ratio_1",
        "BLRegionList": [
            r"fluid-tet-4",
            r"fluid-tet-3",
            r"fluid-tet-2",
            r"fluid-tet-1",
            r"fluid-sweep-fin2",
            r"fluid-sweep-fin1",
            r"fluid-sweep-fin5",
            r"fluid-sweep-fin3",
            r"fluid-sweep-fin6",
            r"fluid-sweep-fin4",
            r"fluid-in",
            r"fluid-out",
        ],
        r"BLZoneList": [
            r"wall-fluid-tet-4-solid-tet-4",
            r"wall-fluid-tet-3-solid-tet-3",
            r"wall-fluid-tet-2-solid-tet-2",
            r"wall-fluid-tet-2-solid-tet-2-wall-fluid-tet-3-solid-tet-3-fluid-tet-2-solid-tet-2",
            r"wall-fluid-tet-1-solid-tet-1",
            r"wall-fluid-sweep-fin-solid-sweep-fin.1",
            r"wall-fluid-sweep-fin-solid-sweep-fin",
            r"wall-fluid-sweep-fin-solid-sweep-fin.5",
            r"wall-fluid-sweep-fin-solid-sweep-fin.4",
            r"wall-fluid-sweep-fin-solid-sweep-fin.3",
            r"wall-fluid-sweep-fin-solid-sweep-fin.2",
        ],
        r"BlLabelList": r"wall*",
        r"CompleteBlLabelList": [
            r"wall-fluid-sweep-fin-solid-sweep-fin",
            r"wall-fluid-tet-1-solid-tet-1",
            r"wall-fluid-tet-2-solid-tet-2",
            r"wall-fluid-tet-3-solid-tet-3",
            r"wall-fluid-tet-4-solid-tet-4",
        ],
        r"FaceScope": {
            r"GrowOn": r"selected-labels",
        },
        r"OffsetMethodType": r"aspect-ratio",
    }
)

meshing.workflow.TaskObject["aspect-ratio_1"].Execute()

#############################################################################
# Generate Mesh
# =============

meshing.workflow.TaskObject["Generate the Volume Mesh"].Execute()

#############################################################################
# Improve Volume Mesh
# ===================

meshing.workflow.TaskObject["Generate the Volume Mesh"].InsertNextTask(
    CommandName=r"ImproveVolumeMesh"
)

meshing.workflow.TaskObject["Improve Volume Mesh"].Arguments.setState(
    {
        r"CellQualityLimit": 0.05,
        r"VMImprovePreferences": {
            r"ShowVMImprovePreferences": False,
            r"VIQualityIterations": 5,
            r"VIQualityMinAngle": 0,
            r"VIgnoreFeature": r"yes",
        },
    }
)

meshing.workflow.TaskObject["Improve Volume Mesh"].Execute()

#############################################################################
# Save Mesh File
# ==============

save_mesh_as = str(Path(pyfluent.EXAMPLES_PATH) / "hx-fin-2mm.msh.h5")
meshing.tui.file.write_mesh(save_mesh_as)

#############################################################################
# Switch to Solution Mode
# =======================

solver = meshing.switch_to_solver()

#############################################################################
# Auto-create Mesh Interfaces
# ===========================

solver.tui.define.mesh_interfaces.create("int", "yes", "no")

#############################################################################
# Mesh Check; Review Fluent transcript for errors
# ===============================================

solver.mesh.check()

#############################################################################
# Create a few boundary list for display and post-processing
# ==========================================================

graphics_session1 = Graphics(solver)
mesh1 = graphics_session1.Meshes["mesh-1"]

wall_list = []
periodic_list = []
symmetry_list = []

for item in mesh1.surfaces_list.allowed_values:
    if len(item.split("wall")) > 1:
        wall_list.append(item)
    if len(item.split("periodic")) > 1:
        periodic_list.append(item)
    if len(item.split("symmetry")) > 1:
        symmetry_list.append(item)

#############################################################################
# Display Mesh
# ============

mesh1.show_edges = True
mesh1.surfaces_list = wall_list
mesh1.display("window-1")
p = pyvista_windows_manager.get_plotter("window-1")
p.view_isometric()
p.add_axes()
p.add_floor(offset=1, show_edges=False)
light = pv.Light(light_type="headlight")
p.add_light(light)

###############################################################################
# Temperature, Energy, Laminar Viscous Model
# ==========================================
# * Set Temperature Unit
# * Enable Energy Equation
# * Enable Laminar Viscous Model

solver.setup.general.units.set_units(
    quantity="temperature", units_name="c", scale_factor=1.0, offset=0.0
)
solver.setup.models.energy.enabled = True
solver.setup.models.viscous.model.set_state("laminar")

#############################################################################
# Change a few material properties of default Air
# ===============================================

air_dict = solver.setup.materials.fluid["air"].get_state()
air_dict["density"]["value"] = 1.2
air_dict["viscosity"]["value"] = 1.5e-5
air_dict["thermal_conductivity"]["value"] = 0.026
air_dict["specific_heat"]["value"] = 1006.0
solver.setup.materials.fluid["air"].set_state(air_dict)

#############################################################################
# Change a few material properties of default Aluminum
# ====================================================

al_dict = solver.setup.materials.solid["aluminum"].get_state()
al_dict["density"]["value"] = 2719.0
al_dict["thermal_conductivity"]["value"] = 200.0
al_dict["specific_heat"]["value"] = 871.0
solver.setup.materials.solid["aluminum"].set_state(al_dict)

#############################################################################
# Copy Copper and change a few material properties of default Copper
# ==================================================================

solver.setup.materials.database.copy_by_name(type="solid", name="copper")
cu_dict = solver.setup.materials.solid["copper"].get_state()
cu_dict["density"]["value"] = 8978.0
cu_dict["thermal_conductivity"]["value"] = 340.0
cu_dict["specific_heat"]["value"] = 381.0
solver.setup.materials.solid["copper"].set_state(cu_dict)

#############################################################################
# Set Tube Cell Zone Material as Copper
# =====================================

tube_dict = solver.setup.cell_zone_conditions.solid["solid-tube-1"].get_state()
tube_dict["material"] = "copper"
solver.setup.cell_zone_conditions.solid["solid-tube-1"].set_state(tube_dict)

tube_dict = solver.setup.cell_zone_conditions.solid["solid-tube-2"].get_state()
tube_dict["material"] = "copper"
solver.setup.cell_zone_conditions.solid["solid-tube-2"].set_state(tube_dict)

#############################################################################
# Set Boundary Condition for Inlet and Outlet
# ===========================================

solver.setup.boundary_conditions.velocity_inlet["inlet"].momentum.velocity = 4.0
solver.setup.boundary_conditions.velocity_inlet["inlet"].thermal.temperature = (
    293.15  # Need to specify in Kelvin
)

solver.setup.boundary_conditions.pressure_outlet[
    "outlet"
].thermal.backflow_total_temperature = 293.15

#############################################################################
# Set Thermal Boundary Condition for Wall Inner Tube
# ==================================================

solver.setup.boundary_conditions.wall["wall-inner-tube-1"].thermal.thermal_condition = (
    "Convection"
)
solver.setup.boundary_conditions.wall[
    "wall-inner-tube-1"
].thermal.heat_transfer_coeff = 1050.0
solver.setup.boundary_conditions.wall["wall-inner-tube-1"].thermal.free_stream_temp = (
    353.15
)

solver.setup.boundary_conditions.copy(from_="wall-inner-tube-1", to="wall-inner-tube-2")

#############################################################################
# Enable HOTR
# ===========

solver.solution.methods.high_order_term_relaxation.enable = True

#############################################################################
# Define Report Definitions
# =========================

solver.solution.report_definitions.surface["outlet-enthalpy-flow"] = {}
solver.solution.report_definitions.surface["outlet-enthalpy-flow"].report_type = (
    "surface-flowrate"
)
solver.solution.report_definitions.surface["outlet-enthalpy-flow"].field = "enthalpy"
solver.solution.report_definitions.surface["outlet-enthalpy-flow"].surface_names = [
    "outlet"
]

solver.solution.report_definitions.surface["avg-pressure-inlet"] = {}
solver.solution.report_definitions.surface["avg-pressure-inlet"].report_type = (
    "surface-areaavg"
)
solver.solution.report_definitions.surface["avg-pressure-inlet"].field = "pressure"
solver.solution.report_definitions.surface["avg-pressure-inlet"].surface_names = [
    "inlet"
]

solver.solution.report_definitions.volume["max-vel-louvers4"] = {}
solver.solution.report_definitions.volume["max-vel-louvers4"].report_type = "volume-max"
solver.solution.report_definitions.volume["max-vel-louvers4"].field = (
    "velocity-magnitude"
)
solver.solution.report_definitions.volume["max-vel-louvers4"].cell_zones = [
    "fluid-tet-4"
]

solver.solution.report_definitions.surface["wall-shear-int"] = {}
solver.solution.report_definitions.surface["wall-shear-int"].report_type = (
    "surface-integral"
)
solver.solution.report_definitions.surface["wall-shear-int"].field = "wall-shear"
solver.solution.report_definitions.surface["wall-shear-int"].surface_names = [
    "wall-fluid-sweep-fin-solid-sweep-fin-shadow",
    "wall-fluid-tet-1-solid-tet-1",
    "wall-fluid-tet-2-solid-tet-2",
    "wall-fluid-tet-3-solid-tet-3",
    "wall-fluid-tet-4-solid-tet-4",
]

solver.solution.monitor.report_plots.create(name="outlet-enthalpy-flow-plot")
solver.solution.monitor.report_plots["outlet-enthalpy-flow-plot"].report_defs = (
    "outlet-enthalpy-flow"
)

solver.solution.monitor.report_files["outlet-enthalpy-flow-file"] = {}
solver.solution.monitor.report_files["outlet-enthalpy-flow-file"] = {
    "report_defs": ["outlet-enthalpy-flow"],
    "file_name": r"outlet-enthalpy-flow.out",
}

solver.solution.monitor.report_plots["avg-pressure-inlet-plot"] = {}
solver.solution.monitor.report_plots["avg-pressure-inlet-plot"] = {
    "report_defs": ["avg-pressure-inlet"]
}

solver.solution.monitor.report_files["avg-pressure-inlet-file"] = {}
solver.solution.monitor.report_files["avg-pressure-inlet-file"] = {
    "report_defs": ["avg-pressure-inlet"],
    "file_name": r"avg-pressure-inlet.out",
}

solver.solution.monitor.report_plots["max-vel-louvers4-plot"] = {}
solver.solution.monitor.report_plots["max-vel-louvers4-plot"] = {
    "report_defs": ["max-vel-louvers4"]
}

solver.solution.monitor.report_files["max-vel-louvers4-file"] = {}
solver.solution.monitor.report_files["max-vel-louvers4-file"] = {
    "report_defs": ["max-vel-louvers4"],
    "file_name": r"max-vel-louvers4.out",
}

solver.solution.monitor.report_plots["wall-shear-int-plot"] = {}
solver.solution.monitor.report_plots["wall-shear-int-plot"] = {
    "report_defs": ["wall-shear-int"]
}

solver.solution.monitor.report_files["wall-shear-int-file"] = {}
solver.solution.monitor.report_files["wall-shear-int-file"] = {
    "report_defs": ["wall-shear-int"],
    "file_name": r"wall-shear-int.out",
}

#############################################################################
# Hybrid Initialization; Slit Interior between Solid Zones; Save Case
# ===================================================================

solver.solution.initialization.initialization_type = "hybrid"
solver.solution.initialization.hybrid_initialize()

solver.setup.boundary_conditions.slit_interior_between_diff_solids()
solver.file.write(file_type="case", file_name="hx-fin-2mm.cas.h5")

#############################################################################
# Set Aggressive Length Scale Method; Run Calculation & Save Data
# ===============================================================

solver.solution.run_calculation.pseudo_time_settings.time_step_method.time_step_method = (
    "automatic"
)
solver.solution.run_calculation.pseudo_time_settings.time_step_method.length_scale_methods = (
    "aggressive"
)

solver.solution.run_calculation.iterate(iter_count=250)

solver.file.write(file_type="case-data", file_name="hx-fin-2mm.dat.h5")

#############################################################################
# Post-Processing Mass Balance Report
# ===================================

inlet_mfr = solver.scheme_eval.exec(
    ('(ti-menu-load-string "/report/fluxes/mass-flow no inlet () no")',)
).split(" ")[-1]
outlet_mfr = solver.scheme_eval.exec(
    ('(ti-menu-load-string "/report/fluxes/mass-flow no outlet () no")',)
).split(" ")[-1]
net_mfr = solver.scheme_eval.exec(
    ('(ti-menu-load-string "/report/fluxes/mass-flow no inlet outlet () no")',)
).split(" ")[-1]
print("Mass Balance Report\n")
print("Inlet (kg/s): ", inlet_mfr)
print("Outlet (kg/s): ", outlet_mfr)
print("Net (kg/s): ", net_mfr)

#############################################################################
# Heat Balance Report
# ===================

htr = solver.scheme_eval.exec(
    ('(ti-menu-load-string "/report/fluxes/heat-transfer yes no")',)
).split(" ")[-1]
print("Heat Balance Report\n")
print("Net Imbalance (Watt): ", htr)

#############################################################################
# Plot Monitors
# =============

fig, axs = plt.subplots(2, 2, figsize=(10, 8))
fig.suptitle("Monitor Plots")

outFilesList = []
fileList = os.listdir(os.getcwd())
for tempFile in fileList:
    fName, ext = os.path.splitext(tempFile)
    if ext == ".out":
        outFilesList.append(tempFile)
outFilesList.sort()

index = 0
for ax in axs.flat:
    X = []
    Y = []
    i = -1
    with open(outFilesList[index], "r") as datafile:
        plotting = csv.reader(datafile, delimiter=" ")
        for rows in plotting:
            i += 1
            if i == 1:
                var = rows[1]
            if i > 2:
                X.append(int(rows[0]))
                Y.append(float(rows[1]))

    ax.plot(X, Y)
    ax.set(xlabel="Iteration", ylabel=var, title=var)
    index += 1

plt.tight_layout()

#############################################################################
# Show graph
# ==========

plt.show()

# %%
# .. image:: ../../_static/cht_avg_pressure.png
#    :align: center
#    :alt: Average Pressure

# %%
#    Average Pressure

#############################################################################
# Contour Plot
# ============

graphics_session1 = Graphics(solver)
contour1 = graphics_session1.Contours["contour-1"]
contour1.field = "temperature"
contour1.surfaces_list = wall_list
contour1.display("window-2")

p = pyvista_windows_manager.get_plotter("window-2")
p.view_isometric()
p.add_axes()
p.add_floor(offset=1, show_edges=False)
p.add_title(
    "Contour of Temperature on Walls", font="courier", color="grey", font_size=10
)
light = pv.Light(light_type="headlight")
p.add_light(light)

p.remove_scalar_bar()
p.add_scalar_bar(
    "Temperature [K]",
    interactive=True,
    vertical=False,
    title_font_size=20,
    label_font_size=15,
    outline=False,
    position_x=0.5,
    fmt="%10.1f",
)

#############################################################################
# Create Iso-Surface of X=0.012826 m
# ==================================

solver.results.surfaces.iso_surface["x=0.012826"] = {}
solver.results.surfaces.iso_surface["x=0.012826"].field = "x-coordinate"
solver.results.surfaces.iso_surface["x=0.012826"] = {"iso_values": [0.012826]}

#############################################################################
# Vecotor Plot
# ============

graphics_session1 = Graphics(solver)
vector1 = graphics_session1.Vectors["vector-1"]
vector1.surfaces_list = ["x=0.012826"]
vector1.scale = 2.0
vector1.skip = 5
vector1.display("window-3")

p = pyvista_windows_manager.get_plotter("window-3")
p.view_isometric()
p.add_axes()
# p.add_floor( offset=1, show_edges=False)
p.add_title("Vector Plot", font="courier", color="grey", font_size=10)
light = pv.Light(light_type="headlight")
p.add_light(light)

p.remove_scalar_bar()
p.add_scalar_bar(
    "Velocity [m/s]",
    interactive=True,
    vertical=False,
    title_font_size=20,
    label_font_size=15,
    outline=False,
    position_x=0.5,
    fmt="%10.1f",
)

o = PyVistaWindow(None, None)
o._fetch_mesh(mesh1)
o._display_mesh(mesh1, p)

#############################################################################
# XY Plot of Pressure
# ===================

plots_session1 = Plots(solver)
p1 = plots_session1.XYPlots["p1"]
p1.surfaces_list = ["x=0.012826"]
p1.y_axis_function = "pressure"
p1.x_axis_function = "direction-vector"
p1.direction_vector.set_state([0, 1, 0])

#############################################################################
# Show graph
# ==========

p1.plot("p1")

# %%
# .. image:: ../../_static/cht_xy_pressure.png
#    :align: center
#    :alt: XY Plot of Pressure

# %%
#    XY Plot of Pressure

#############################################################################
# Exit Fluent Session
# ===================
solver.exit()



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import time

from ansys.fluent.core import connect_to_fluent
from ansys.fluent.core.utils.fluent_version import FluentVersion


def transcript(data):
    transcript.data = data


def run_transcript(i, ip, port, password):
    transcript("")
    session = connect_to_fluent(
        ip=ip, port=port, password=password, cleanup_on_exit=False
    )
    session.transcript.register_callback(transcript)

    transcript_checked = False
    transcript_passed = False

    if i % 5 == 0:
        time.sleep(0.5)
        session.scheme_eval.scheme_eval("(pp 'test)")
        time.sleep(0.5)
        if not transcript.data:
            assert transcript.data == ""
        else:
            assert transcript.data == "test"
            transcript_passed = True
        transcript_checked = True

    return transcript_checked, transcript_passed


def test_transcript(new_solver_session):
    solver = new_solver_session
    ip = solver.connection_properties.ip
    port = solver.connection_properties.port
    password = solver.connection_properties.password

    total_checked_transcripts = 0
    total_passed_transcripts = 0

    for i in range(100):
        transcript_checked, transcript_passed = run_transcript(i, ip, port, password)
        total_checked_transcripts += int(transcript_checked)
        total_passed_transcripts += int(transcript_passed)

    if solver.get_fluent_version() >= FluentVersion.v232:
        assert total_checked_transcripts == total_passed_transcripts
    else:
        assert total_checked_transcripts >= total_passed_transcripts



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""".. _ref_ts_mode_calculator:

Tyler-Sofrin Compressor Modes Post-Processing
---------------------------------------------
"""

#######################################################################################
# Objective
# ~~~~~~~~~
#
# This example demonstrates PyFluent API's for
#
# * Read a case and data file
# * Create monitor points to calculate Fourier coefficients
# * Write Fourier coefficients to a file
# * Tyler-Sofrin mode Plot using the matplotlib library
#
# Background
# ~~~~~~~~~~
#
# Tyler and Sofrin (1961) demonstrated that interactions between a rotor and a
# stator result in an infinite set of spinning modes. Each Tyler-Sofrin (TS)
# mode exhibits an m-lobed pattern and rotates at a speed given by the
# following equation:
#
# :math:`\text{speed} = \frac{BnΩ}{m}`
# Where:
#
# * m is the Tyler-Sofrin mode number, defined as 'm = nB + kV'
# * n is the impeller frequency harmonic
# * k is the vane harmonic
# * B is the number of rotating blades
# * V is the number of stationary vanes
# * Ω is the Rotor shaft speed, rad/s
#
# Example:
#
#         * 8-blade rotor interacting with a 6-vane stator
#         * 2-lobed pattern turning at (8)(1)/(2) = 4 times shaft speed
#
#
# Example Table
# ~~~~~~~~~~~~~
#
# .. image:: ../../_static/ExampleTable.jpg
#    :alt: Example Table
#
#
# Tyler-Sofrin Modes
# ~~~~~~~~~~~~~~~~~~
#
# .. image:: ../../_static/TSmode.jpg
#    :alt: Tyler-Sofrin Modes


#######################################################################################
# Example Note: Discrete Fourier Transform (DFT)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   + In order to calculate the pressure related to each TS-mode, extend the
#     simulation and perform the DFT of pressure at the desired blade passing
#     frequency harmonics.
#
#   + Disable the Hanning windowing (specifically for periodic flows like
#     this one) to avoid getting half the expected magnitudes for periodic flows.
#     Make sure to set the windowing parameter to 'None' when specifying
#     the Discrete Fourier Transform (DFT) in the graphical user interface (GUI).
#
#   + The DFT data will only be accurate if the sampling is done across the
#     entire specified sampling period.
#
#
# .. note::
#   The .cas/.dat file provided with this example is for demonstration purposes only.
#   A finer mesh is necessary for accurate acoustic analysis. This example uses data
#   sets generated with Ansys Fluent V2023R2.

#######################################################################################
# Post-Processing Implementation
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


#######################################################################################
# Import required libraries/modules
# =====================================================================================
import math
from pathlib import Path
import random

import matplotlib.pyplot as plt
import numpy as np

import ansys.fluent.core as pyfluent
from ansys.fluent.core import examples

#######################################################################################
# Specifying save path
# =====================================================================================
# save_path can be specified as:
#
#   +   Path("E:/", "pyfluent-examples-tests") or
#   +   Path("E:/pyfluent-examples-tests") in a Windows machine for example, or
#   +   Path("~/pyfluent-examples-tests") in Linux.

save_path = Path(pyfluent.EXAMPLES_PATH)

#######################################################################################
# Downloading cas/dat file
# =====================================================================================
import_filename = examples.download_file(
    "axial_comp_fullWheel_DFT_23R2.cas.h5",
    "pyfluent/examples/Tyler-Sofrin-Modes-Compressor",
    save_path=save_path,
)

examples.download_file(
    "axial_comp_fullWheel_DFT_23R2.dat.h5",
    "pyfluent/examples/Tyler-Sofrin-Modes-Compressor",
    save_path=save_path,
)

#######################################################################################
# Launch Fluent session and print Fluent version
# =====================================================================================
session = pyfluent.launch_fluent(
    ui_mode="gui", processor_count=4, product_version="25.1.0"
)
print(session.get_fluent_version())

#######################################################################################
# Reading case and data file
# =====================================================================================
#
# .. note::
#   The dat file should correspond to the already completed DFT simulation.

session.file.read(file_type="case-data", file_name=import_filename)

#######################################################################################
# Define User constant/variables
# =====================================================================================
#
# .. note::
#   The variable names should match the ones written from the DFT and can be
#   identified by manually examining the solution variables as shown below:
#
# .. image:: ../../_static/var_names.jpg
#    :alt: variable names

varname = [
    "mean-static-pressure-dataset",
    "dft-static-pressure_10.00kHz-ta",
    "dft-static-pressure-1_21.43kHz-ta",
    "dft-static-pressure-2_30.00kHz-ta",
]
n_mode = [0, 1, 2, 3]  # Impeller frequency harmonics
r = 0.082  # meters
z = -0.037  # meters
d_theta = 5  # degrees
m_max = 50  # maximum TS mode number

# Plot will be from -m_max to +m_max, incremented by m_inc
m_inc = 2  # TS mode increment

#######################################################################################
# Create monitor points
# =====================================================================================
for angle in range(0, 360, d_theta):
    x = math.cos(math.radians(angle)) * r
    y = math.sin(math.radians(angle)) * r
    pt_name = "point-" + str(angle)
    session.results.surfaces.point_surface[pt_name] = {}
    session.results.surfaces.point_surface[pt_name].point = [x, y, z]

#######################################################################################
# Compute Fourier coefficients at each monitor point (An, Bn)
# =====================================================================================
An = np.zeros((len(varname), int(360 / d_theta)))
Bn = np.zeros((len(varname), int(360 / d_theta)))

for angle_ind, angle in enumerate(range(0, 360, d_theta)):
    for n_ind, variable in enumerate(varname):
        if len(variable) >= 4 and variable[:4] == "mean":
            session.solution.report_definitions.surface["mag-report"] = {
                "report_type": "surface-vertexavg",
                "surface_names": ["point-" + str(angle)],
                "field": str(variable),
            }
            mag = session.solution.report_definitions.compute(
                report_defs=["mag-report"]
            )
            mag = mag[0]["mag-report"][0]
            An[n_ind][angle_ind] = mag
            Bn[n_ind][angle_ind] = 0
        else:
            session.solution.report_definitions.surface["mag-report"] = {
                "report_type": "surface-vertexavg",
                "surface_names": ["point-" + str(angle)],
                "field": str(variable) + "-mag",
            }
            mag = session.solution.report_definitions.compute(
                report_defs=["mag-report"]
            )
            mag = mag[0]["mag-report"][0]
            session.solution.report_definitions.surface["phase-report"] = {
                "report_type": "surface-vertexavg",
                "surface_names": ["point-" + str(angle)],
                "field": str(variable) + "-phase",
            }
            phase = session.solution.report_definitions.compute(
                report_defs=["phase-report"]
            )
            phase = phase[0]["phase-report"][0]
            An[n_ind][angle_ind] = mag * math.cos(phase)
            Bn[n_ind][angle_ind] = -mag * math.sin(phase)


#######################################################################################
# Write Fourier coefficients to file
# =====================================================================================
#
# .. note::
#   This step is only required if data is to be processed with other standalone
#   tools. Update the path to the file accordingly.

fourier_coefficients_file = Path(save_path, "FourierCoefficients.txt")
with open(fourier_coefficients_file, "w") as f:
    f.write("n theta An Bn \n")

    for n_ind, variable in enumerate(varname):
        for ind, x in enumerate(An[n_ind, :]):
            f.write(
                str(n_mode[n_ind])
                + ","
                + str(ind * d_theta)
                + ","
                + str(An[n_ind, ind])
                + ","
                + str(Bn[n_ind, ind])
                + "\n"
            )


#######################################################################################
# Calculate Resultant Pressure Field
# =====================================================================================
#
# Create list of m values based on m_max and m_inc
#
# .. image:: ../../_static/TS_formulas.jpg
#    :alt: variable names

m_mode = range(-m_max, m_max + m_inc, m_inc)

# Initialize solution matrices with zeros
Anm = np.zeros((len(varname), len(m_mode)))
Bnm = np.zeros((len(varname), len(m_mode)))
Pnm = np.zeros((len(varname), len(m_mode)))

for n_ind, variable in enumerate(varname):  # loop over n modes
    for m_ind, m in enumerate(m_mode):  # loop over m modes
        for angle_ind, angle in enumerate(
            np.arange(0, math.radians(360), math.radians(d_theta))
        ):  # loop over all angles, in radians
            Anm[n_ind][m_ind] += An[n_ind][angle_ind] * math.cos(m * angle) - Bn[n_ind][
                angle_ind
            ] * math.sin(m * angle)
            Bnm[n_ind][m_ind] += An[n_ind][angle_ind] * math.sin(m * angle) + Bn[n_ind][
                angle_ind
            ] * math.cos(m * angle)
        Anm[n_ind][m_ind] = Anm[n_ind][m_ind] / (2 * math.pi) * math.radians(d_theta)
        Bnm[n_ind][m_ind] = Bnm[n_ind][m_ind] / (2 * math.pi) * math.radians(d_theta)
        Pnm[n_ind][m_ind] = math.sqrt(Anm[n_ind][m_ind] ** 2 + Bnm[n_ind][m_ind] ** 2)

# P_00 is generally orders of magnitude larger than that of other modes.
# Giving focus to other modes by setting P_00 equal to zero
Pnm[0][int(len(m_mode) / 2)] = 0

#######################################################################################
# Plot Tyler-Sofrin modes
# =====================================================================================
#
fig = plt.figure()
ax = plt.axes(projection="3d")
ax.set_xlabel("Tyler-Sofrin Mode, m")
ax.set_ylabel("Imp Freq Harmonic, n")
ax.set_zlabel("Pnm [Pa]")
plt.yticks(n_mode)
for n_ind, n in enumerate(n_mode):
    x = m_mode
    y = np.full(Pnm.shape[1], n)
    z = Pnm[n_ind]
    rgb = (random.random(), random.random(), random.random())
    ax.plot3D(x, y, z, c=rgb)
plt.show()

#######################################################################################
# Tyler-Sofrin modes
# =====================================================================================
# .. image:: ../../_static/ts_modes.png
#    :alt: Tyler-Sofrin modes


#######################################################################################
# Close the session
# =====================================================================================
session.exit()


#######################################################################################
# References
# =====================================================================================
#
# [1] J.M. Tyler and  T. G. Sofrin, Axial Flow Compressor Noise Studies,1961 Manly
# Memorial Award.

# sphinx_gallery_thumbnail_path = '_static/ts_modes.png'
